Ext.namespace('Zarafa.plugins.owncloud');

/**
 * @class Zarafa.plugins.owncloud.OwncloudMainPlugin
 * @extends Zarafa.core.Plugin
 * This cplugin is used to load the settings widget!
 */
Zarafa.plugins.owncloud.OwncloudMainPlugin = Ext.extend(Zarafa.core.Plugin, {
	/**
	 * @constructor
	 * @param {Object} config
	 */
	constructor : function(config)
	{
		config = config || {};
		Ext.applyIf(config, {
			name : 'owncloud'
		});
		
		Zarafa.plugins.owncloud.OwncloudMainPlugin.superclass.constructor.call(this, config);
	},

	/**
	 * initialises insertion point for plugin
	 * @protected
	 */
	initPlugin : function()
	{
		Zarafa.plugins.owncloud.OwncloudMainPlugin.superclass.initPlugin(this, arguments);		
		
		/* settingswidget */
		/* Register the Mail category for the settings */
		this.registerInsertionPoint('context.settings.categories', this.createSettingCategories, this);
		
		/* use zarafa user as default user */
		if(container.getSettingsModel().get('zarafa/v1/contexts/owncloud/username') == "defaultusername") {
			container.getSettingsModel().set('zarafa/v1/contexts/owncloud/username', container.getUser().getUserName());
		}
	},
	
	/**
	 * Create the owncloud {@link Zarafa.settings.ui.SettingsCategory Settings Category}
	 * to the {@link Zarafa.settings.SettingsContext}. This will create new
	 * {@link Zarafa.settings.ui.SettingsCategoryTab tabs} for the
	 * {@link Zarafa.owncloud.settings.SettingsOwncloudCategory Owncloud} and the 
	 * in the {@link Zarafa.settings.ui.SettingsCategoryWidgetPanel Widget Panel}.
	 * @return {Array} configuration object for the categories to register
	 * @private
	 */
	createSettingCategories: function()
	{
		return [{
			xtype : 'zarafa.settingsowncloudcategory'
		}]
	}
});


Ext.namespace('Zarafa.plugins.owncloudattachment');
/**
 * @class Zarafa.plugins.owncloudattachment.OwncloudAttachmentPlugin
 * @extends Zarafa.core.Plugin
 * This class is used for adding files from the users's Owncloud folder
 * to his eownclouds as attachments
 */
Zarafa.plugins.owncloudattachment.OwncloudAttachmentPlugin = Ext.extend(Zarafa.core.Plugin, {

	/**
	 * @constructor
	 * @param {Object} config
	 */
	constructor : function(config)
	{
		config = config || {};
		Ext.applyIf(config, {
			name : 'owncloudattachment'
		});
		
		Zarafa.plugins.owncloudattachment.OwncloudAttachmentPlugin.superclass.constructor.call(this, config);
	},

	/**
	 * initialises insertion point for plugin
	 * @protected
	 */
	initPlugin : function()
	{
		Zarafa.plugins.owncloudattachment.OwncloudAttachmentPlugin.superclass.initPlugin(this, arguments);
		
		/* load attachments from owncloud */
		this.registerInsertionPoint('main.attachment.method', this.onAttachmentInsertion)
		Zarafa.core.data.SharedComponentType.addProperty('common.dialog.attachments.owncloud');
	},

	/**
	 * Insert owncloud option in all attachment suggestions
	 */
	onAttachmentInsertion : function(include, btn) {
		return {
			text : _('Owncloud'),
			handler : this.showOwncloudAttachmentDialog.createDelegate(this, [btn]),
			scope: this,
			iconCls: 'icon_owncloud'
		};
	},

	/**
	 * Initializes Dialog for adding attachment from Owncloud to eowncloud
	 * @param {Object} btn
	 * @private
	 */
	showOwncloudAttachmentDialog : function(btn)
	{
		Zarafa.core.data.UIFactory.openLayerComponent(Zarafa.core.data.SharedComponentType['common.dialog.attachments.owncloud'], btn.record, {
			title : _('Add attachment from Owncloud'),
			manager : Ext.WindowMgr
		});
	},
	
	/**
	 * Bid for the type of shared component
	 * and the given record.
	 * This will bid on calendar.dialogs.importevents
	 * @param {Zarafa.core.data.SharedComponentType} type Type of component a context can bid for.
	 * @param {Ext.data.Record} record Optionally passed record.
	 * @return {Number} The bid for the shared component
	 */
	bidSharedComponent : function(type, record)
	{
		var bid = -1;
		
		switch(type) {
			case Zarafa.core.data.SharedComponentType['common.dialog.attachments.owncloud']:
				if (record instanceof Zarafa.core.data.IPMRecord) {
					if (record.supportsAttachments()) {
						bid = 1;
					}
				}
				break;
		}
		return bid;
	},

	/**
	 * Will return the reference to the shared component.
	 * Based on the type of component requested a component is returned.
	 * @param {Zarafa.core.data.SharedComponentType} type Type of component a context can bid for.
	 * @param {Ext.data.Record} record Optionally passed record.
	 * @return {Ext.Component} Component
	 */
	getSharedComponent : function(type, record)
	{
		var component;

		switch(type) {
			case Zarafa.core.data.SharedComponentType['common.dialog.attachments.owncloud']:
				component = Zarafa.plugins.owncloudattachment.dialogs.AttachmentContentPanel;
				break;
		}

		return component;
	}	
});
Ext.namespace('Zarafa.plugins.owncloudrcvattachment');

/**
 * @class Zarafa.plugins.owncloudattachment.OwncloudRcvAttachmentPlugin
 * @extends Zarafa.core.Plugin
 * This class is used for adding files from the users's Owncloud folder
 * to his eownclouds as attachments
 */
Zarafa.plugins.owncloudattachment.OwncloudRcvAttachmentPlugin = Ext.extend(Zarafa.core.Plugin, {

	/**
	 * @constructor
	 * @param {Object} config
	 */
	constructor : function(config)
	{
		config = config || {};
		Ext.applyIf(config, {
			name : 'owncloudrcvattachment'
		});
		
		Zarafa.plugins.owncloudattachment.OwncloudRcvAttachmentPlugin.superclass.constructor.call(this, config);
	},

	/**
	 * initialises insertion point for plugin
	 * @protected
	 */
	initPlugin : function()
	{
		Zarafa.plugins.owncloudattachment.OwncloudRcvAttachmentPlugin.superclass.initPlugin(this, arguments);
		
		/* store attachments in owncloud */
		this.registerInsertionPoint('common.contextmenu.attachment.actions', this.onAttachmentInsertion);
		Zarafa.core.data.SharedComponentType.addProperty('common.dialog.attachments.savetoowncloud');
	},
	
	/**
	 * called on successfull preload of attachment
	 */
	onSuccessPrepare : function(response)
	{
		Zarafa.common.dialogs.MessageBox.hide();
		Zarafa.core.data.UIFactory.openLayerComponent(Zarafa.core.data.SharedComponentType['common.dialog.attachments.savetoowncloud'], response, {
			title : _('Add attachment to Owncloud'),
			manager : Ext.WindowMgr
		});
	},

	/**
	 * Insert owncloud option in all attachment suggestions
	 */
	onAttachmentInsertion : function(include, btn) {
		return {
			text : _('Owncloud'),
			handler : this.prepareAttachment.createDelegate(this, [btn,this.onSuccessPrepare]),
			scope: this,
			iconCls: 'icon_owncloud'
		};
	},

	/**
	 * prepareAttachment
	 * @param {Object} btn
	 * @private
	 */
	prepareAttachment : function(btn,callback)
	{
	
		Zarafa.common.dialogs.MessageBox.show({
			title: 'Please wait',
			msg: 'Loading attachment...',
			progressText: 'Initializing...',
			width:300,
			progress:true,
			closable:false
		});

		// progress bar... ;)
		var f = function(v){
			return function(){
				if(v == 100){
					Zarafa.common.dialogs.MessageBox.hide();
				}else{
					Zarafa.common.dialogs.MessageBox.updateProgress(v/100, Math.round(v)+'% loaded');
				}
		   };
		};
		
		for(var i = 1; i < 101; i++){
			setTimeout(f(i), 20*i);
		}
		
		/* store the attachment to a temporary folder and prepare it for uploading */
		var attachmentRecord = btn.records;
		var attachmentStore = attachmentRecord.store;
		
		var store = attachmentStore.getParentRecord().get('store_entryid');
		var entryid = attachmentStore.getAttachmentParentRecordEntryId();
		var attachNum = new Array(1);		
		if (attachmentRecord.get('attach_num') != -1)
			attachNum[0] = attachmentRecord.get('attach_num');
		else
			attachNum[0] = attachmentRecord.get('tmpname');
		var dialog_attachments = attachmentStore.getId();
		var filename = attachmentRecord.data.name;
		
		var responseHandler = new Zarafa.owncloud.data.ResponseHandler({
			successCallback: callback
		});
		
		// request attachment preperation
		container.getRequest().singleRequest(
			'owncloudmodule',
			'prepare-attachment',
			{
				entryid : entryid,
				store: store,
				attachNum: attachNum,
				dialog_attachments: dialog_attachments,
				filename: filename
			},
			responseHandler
		);
	},
	
	/**
	 * Bid for the type of shared component
	 * and the given record.
	 * This will bid on common.dialog.attachments.savetoowncloud
	 * @param {Zarafa.core.data.SharedComponentType} type Type of component a context can bid for.
	 * @param {Object} response Optionally passed response.
	 * @return {Number} The bid for the shared component
	 */
	bidSharedComponent : function(type, response)
	{
		var bid = -1;
		
		switch(type)
		{
			case Zarafa.core.data.SharedComponentType['common.dialog.attachments.savetoowncloud']:
				if(response.status == true) {
					bid = 1;
				}
				break;
		}
		return bid;
	},

	/**
	 * Will return the reference to the shared component.
	 * Based on the type of component requested a component is returned.
	 * @param {Zarafa.core.data.SharedComponentType} type Type of component a context can bid for.
	 * @param {Object} response Optionally passed response.
	 * @return {Ext.Component} Component
	 */
	getSharedComponent : function(type, response)
	{
		var component;

		switch(type)
		{
			case Zarafa.core.data.SharedComponentType['common.dialog.attachments.savetoowncloud']:
				component = Zarafa.plugins.owncloudrcvattachment.dialogs.FolderChoosePanel;
				break;
		}

		return component;
	}
});

/**
 * imports the older settings model
 */
function importSettings () {
	var settingsModel = container.getSettingsModel();
	
	var Base64 = {
		// private property
		_keyStr : "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",

		// public method for encoding
		encode : function (input) {
			var output = "";
			var chr1, chr2, chr3, enc1, enc2, enc3, enc4;
			var i = 0;

			input = Base64._utf8_encode(input);

			while (i < input.length) {

				chr1 = input.charCodeAt(i++);
				chr2 = input.charCodeAt(i++);
				chr3 = input.charCodeAt(i++);

				enc1 = chr1 >> 2;
				enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
				enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
				enc4 = chr3 & 63;

				if (isNaN(chr2)) {
					enc3 = enc4 = 64;
				} else if (isNaN(chr3)) {
					enc4 = 64;
				}

				output = output +
				this._keyStr.charAt(enc1) + this._keyStr.charAt(enc2) +
				this._keyStr.charAt(enc3) + this._keyStr.charAt(enc4);

			}

			return output;
		},
		
		// private method for UTF-8 encoding
		_utf8_encode : function (string) {
			string = string.replace(/\r\n/g,"\n");
			var utftext = "";

			for (var n = 0; n < string.length; n++) {

				var c = string.charCodeAt(n);

				if (c < 128) {
					utftext += String.fromCharCode(c);
				}
				else if((c > 127) && (c < 2048)) {
					utftext += String.fromCharCode((c >> 6) | 192);
					utftext += String.fromCharCode((c & 63) | 128);
				}
				else {
					utftext += String.fromCharCode((c >> 12) | 224);
					utftext += String.fromCharCode(((c >> 6) & 63) | 128);
					utftext += String.fromCharCode((c & 63) | 128);
				}

			}

			return utftext;
		}
	}
	
	/* check if old settings are present */
	if(settingsModel.get('zarafa/v1/plugins/owncloudattachment/enable') || settingsModel.get('zarafa/v1/plugins/owncloudattachment/enable_store_to_oc')) {
		
		var oldSettings = settingsModel.get('zarafa/v1/plugins/owncloudattachment',true);
		
		/* set old settings as new ones... */
		settingsModel.beginEdit();
		settingsModel.set('zarafa/v1/plugins/owncloud/enable_attach_from_oc', oldSettings['enable']);
		settingsModel.set('zarafa/v1/plugins/owncloud/enable_store_to_oc', oldSettings['enable_store_to_oc']);
		settingsModel.set('zarafa/v1/contexts/owncloud/username', oldSettings['username']);
		settingsModel.set('zarafa/v1/contexts/owncloud/password', Base64.encode(oldSettings['password']));
		settingsModel.set('zarafa/v1/contexts/owncloud/server', oldSettings['owncloud_server']);
		settingsModel.set('zarafa/v1/contexts/owncloud/owncloud_path', oldSettings['owncloud_path']);
		settingsModel.set('zarafa/v1/contexts/owncloud/use_ssl', oldSettings['use_ssl']);
		settingsModel.endEdit();
		
		/* remove old settings */
		settingsModel.remove('zarafa/v1/plugins/owncloudattachment');
		
		settingsModel.save();
	}
}

/**
 * registers plugin
 */
Zarafa.onReady(function() {
	
	importSettings(); // try to upgrade older installations
	container.registerPlugin( new Zarafa.plugins.owncloud.OwncloudMainPlugin);
	
	if(container.getSettingsModel().get('zarafa/v1/plugins/owncloud/enable_attach_from_oc') === true)  {
		container.registerPlugin( new Zarafa.plugins.owncloudattachment.OwncloudAttachmentPlugin);
	}
	if(container.getSettingsModel().get('zarafa/v1/plugins/owncloud/enable_store_to_oc') === true)  {
		container.registerPlugin( new Zarafa.plugins.owncloudattachment.OwncloudRcvAttachmentPlugin);
	}
});
